"""
ZLOGIN Module is for auto-login for Zerodha through retrieval of the access token to dynamoDB.
This works in conjunction with access storage module which creates a Lambda container handling re-direct url from zerodha
Check the repos at zerodha-dynamodb-save-accesstoken, zlogin-puppeteer
"""
import os
import sys
from datetime import datetime
from logging import getLogger, INFO, StreamHandler
import boto3
from kiteconnect import KiteConnect, KiteTicker

logger = getLogger("Zlogin")
logger.addHandler(StreamHandler(sys.stdout))
logger.setLevel(INFO)


def get_env_zapi():
    """Get env vars of the setup - basically to get the Zerodha API key"""
    # add_env_vars_from_file()
    return os.getenv('ZAPI')


def fetch_access_token():
    """
    This function fetches access token either from Dynamodb - if the token exists for today - or raises Exception if no token present for today
    Args:
        None
    Requirements:
        A '.zerodha' file in the home directory with values for ZAPI, ZSECRET, ZAPI_AUTH, ZUSER, ZPASS, ZTOTP
        Sample in the repo
    """
    dynamodb = boto3.resource('dynamodb')
    access_store = dynamodb.Table('access_store')
    today = datetime.utcnow().date().strftime('%Y-%m-%d')
    date_rows = access_store.query(
        KeyConditionExpression=boto3.dynamodb.conditions.Key(
            'date').eq(today),
        ScanIndexForward=False
    )['Items']
    try:
        logger.info("Access Token Returned!")
        return date_rows[0]['access_token']
    except Exception as exp:
        logger.error(
            'Access token not generated - Neither does one exist for today')
        raise exp


def fetch_kiteconnect_instance(api_key=''):
    """Returns KiteConnect instance for further analysis"""
    access_token = fetch_access_token()
    return KiteConnect(api_key or get_env_zapi(), access_token)


def fetch_kiteticker_instance(api_key=''):
    """
    Returns a KiteTicker Object (WebSocket) - that can be used by defining:
        kws.on_ticks = on_ticks
        kws.on_connect = on_connect
        kws.on_close = on_close
    """
    access_token = fetch_access_token()
    return KiteTicker(api_key or get_env_zapi(), access_token)


if __name__ == "__main__":
    print(fetch_access_token())
    print(fetch_kiteconnect_instance())
    print(fetch_kiteticker_instance())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zlogin']

package_data = \
{'': ['*']}

install_requires = \
['boto3', 'kiteconnect', 'pyotp']

setup_kwargs = {
    'name': 'zlogin',
    'version': '0.0.8',
    'description': 'Zerodha Login Automation - Selenium Based',
    'long_description': '# Zerodha Login Automated; This package reads access token stored in dynamodb and provides KiteConnect / KiteTicker instances.\n# This package can only be used in conjunction with a dynamodb access_store that keeps access key stored \n## Context\nThis code is for a python package for automated login to Zerodha. The redirect URL is not handled here - Separate Repos with a container that runs as a lambda function saves the access token to dynamoDB\n\nCheck out zlogin-puppeteer (headless chrome based login to zerodha) [can be hosted for free in Google cloud functions]\nand zerodha-dynamodb-save-accesstoken [AWS lambda container] to save access token to dynamodb from redirect url.\n\n## How to productionalize\n- Get Zerodha (kite.trade) account for API based access to the platform.\n- Save the API key as env variable ZAPI\n\n- Checkout zlogin-puppeteer repository that automates login process (using headless chrome)\n- Check out the zerodha-dynamodb-save-accesstoken repository to set up a lambda function to handle Zerodha redirect url (it saves the access token in the save_access table in dynamodb)\n- Install zlogin package from pypi, "pip install zlogin"\n\n```python\nimport zlogin\naccess_token = zlogin.fetch_access_token()\n```\n\n## This repo is a sister repo with zerodha-dynamodb-save-accesstoken & zlogin-puppeteer tools which have a container code for lambda function that handles storage of access token to dynamodb and puppeteer based Google Cloud Function code to handle login process which redirects to the lambda.\n',
    'author': 'Prabhat Rastogi',
    'author_email': 'prabhatrastogik@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/prabhatrastogik/zlogin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
